@@ -3,8 +3,6 @@ warnings.simplefilter(action='ignore', category=FutureWarning)
 
 import os
 import sys
-# import threading
-# from queue import Queue
 
 import pandas as pd
 from datetime import datetime, timedelta
@@ -15,21 +13,32 @@ import toml
 import random
 import names
 import string
-# import itertools as it
 import redis
 import pathos
 from pathos.pools import ProcessPool
 
 # Local
 from dragg.mpc_calc import MPCCalc, manage_home
-from dragg.redis_client import RedisClient
+import dragg.redis_client as rc
 from dragg.logger import Logger
 
+REDIS_URL = "redis://localhost"
+
 class Aggregator:
-    def __init__(self):
+    """
+    The aggregator combines the power consumption from all homes in the simulation, and manages
+    the simulation of each home (MPCCalc) object in parallel.
+    """
+    def __init__(self, start=None, end=None, redis_url=REDIS_URL):
+        """
+        :parameter start: optional override of the start simulation datetime, string "YYYY-MM-DD HH"
+        :parameter end: optional override of the end simulation datetime, string "YYYY-MM-DD HH"
+        :parameter redis_url: optional override of the Redis host URL (must align with MPCCalc REDIS_URL)
+        """
         self.log = Logger("aggregator")
         self.data_dir = os.path.expanduser(os.environ.get('DATA_DIR','data'))
-        self.outputs_dir = os.path.join('outputs')
+        self.outputs_dir = os.path.join(os.getcwd(), 'outputs')
+        print(self.outputs_dir)
         if not os.path.isdir(self.outputs_dir):
             os.makedirs(self.outputs_dir)
         self.config_file = os.path.join(self.data_dir, os.environ.get('CONFIG_FILE', 'config.toml'))
@@ -45,9 +54,12 @@ class Aggregator:
                 "hems":     {"prediction_horizon", "discomfort", "disutility"}
             },
             "simulation":   {"start_datetime", "end_datetime", "random_seed", "load_zone", "check_type", "run_rbo_mpc"},
-            # "agg":          {"action_horizon", "forecast_horizon", "base_price", "max_rp", "subhourly_steps"}
             "agg":          {"base_price", "subhourly_steps"}
         }
+        
+        self.str_start_dt = start 
+        self.str_end_dt = end
+
         self.timestep = None  # Set by redis_set_initial_values
         self.iteration = None  # Set by redis_set_initial_values
         self.reward_price = None  # Set by redis_set_initial_values
@@ -64,7 +76,8 @@ class Aggregator:
         self.dt = None  # Set by _set_dt
         self.num_timesteps = None  # Set by _set_dt
         self.all_homes = None  # Set by create_homes
-        self.redis_client = RedisClient()
+        self.redis_url = redis_url
+        self.redis_client = rc.connection(redis_url)#RedisClient(redis_url)
         self.config = self._import_config()
         self.check_type = self.config['simulation']['check_type']  # One of: 'pv_only', 'base', 'battery_only', 'pv_battery', 'all'
 
@@ -86,6 +99,8 @@ class Aggregator:
         self.case = "baseline"
         self.start_time = datetime.now()
 
+        self.overwrite_output = False
+
     def _import_config(self):
         if not os.path.exists(self.config_file):
             self.log.logger.error(f"Configuration file does not exist: {self.config_file}")
@@ -115,9 +130,12 @@ class Aggregator:
         objects.  Calculate the number of hours for which the simulation will run.
         :return:
         """
-        try:
-            self.start_dt = datetime.strptime(self.config['simulation']['start_datetime'], '%Y-%m-%d %H')
-            self.end_dt = datetime.strptime(self.config['simulation']['end_datetime'], '%Y-%m-%d %H')
+        try: 
+            if not self.str_start_dt and not self.str_end_dt:
+                self.str_start_dt = self.config["simulation"]["start_datetime"]
+                self.str_end_dt = self.config["simulation"]["end_datetime"]
+            self.start_dt = datetime.strptime(self.str_start_dt, '%Y-%m-%d %H')
+            self.end_dt = datetime.strptime(self.str_end_dt, '%Y-%m-%d %H')
         except ValueError as e:
             self.log.logger.error(f"Error parsing datetimes: {e}")
             sys.exit(1)
@@ -294,6 +312,11 @@ class Aggregator:
             self.config['home']['hvac']['c_dist'][1],
             self.config['community']['total_number_homes']
         )
+        home_w_dist = np.random.uniform(
+            self.config['home']['hvac']['window_eq_dist'][0],
+            self.config['home']['hvac']['window_eq_dist'][1],
+            self.config['community']['total_number_homes']
+        )
         home_hvac_p_cool_dist = np.random.uniform(
             self.config['home']['hvac']['p_cool_dist'][0],
             self.config['home']['hvac']['p_cool_dist'][1],
@@ -434,6 +457,9 @@ class Aggregator:
                 "hvac": {
                     "r": home_r_dist[i],
                     "c": home_c_dist[i],
+                    "w": home_w_dist[i],
+                    "hvac_seer": self.config["home"]["hvac"]["seer"],
+                    "hvac_hspf": self.config["home"]["hvac"]["hspf"],
                     "p_c": home_hvac_p_cool_dist[i],
                     "p_h": home_hvac_p_heat_dist[i],
                     "temp_in_min": home_hvac_temp_in_min_dist[i],
@@ -478,6 +504,9 @@ class Aggregator:
                 "hvac": {
                     "r": home_r_dist[i],
                     "c": home_c_dist[i],
+                    "w": home_w_dist[i],
+                    "hvac_seer": self.config["home"]["hvac"]["seer"],
+                    "hvac_hspf": self.config["home"]["hvac"]["hspf"],
                     "p_c": home_hvac_p_cool_dist[i],
                     "p_h": home_hvac_p_heat_dist[i],
                     "temp_in_min": home_hvac_temp_in_min_dist[i],
@@ -531,6 +560,9 @@ class Aggregator:
                 "hvac": {
                     "r": home_r_dist[i],
                     "c": home_c_dist[i],
+                    "w": home_w_dist[i],
+                    "hvac_seer": self.config["home"]["hvac"]["seer"],
+                    "hvac_hspf": self.config["home"]["hvac"]["hspf"],
                     "p_c": home_hvac_p_cool_dist[i],
                     "p_h": home_hvac_p_heat_dist[i],
                     "temp_in_min": home_hvac_temp_in_min_dist[i],
@@ -567,6 +599,9 @@ class Aggregator:
                 "hvac": {
                     "r": home_r_dist[i],
                     "c": home_c_dist[i],
+                    "w": home_w_dist[i],
+                    "hvac_seer": self.config["home"]["hvac"]["seer"],
+                    "hvac_hspf": self.config["home"]["hvac"]["hspf"],
                     "p_c": home_hvac_p_cool_dist[i],
                     "p_h": home_hvac_p_heat_dist[i],
                     "temp_in_min": home_hvac_temp_in_min_dist[i],
@@ -600,9 +635,9 @@ class Aggregator:
 
     def create_mpc_home_obj(self):
         for home in self.all_homes:
-            home_obj = MPCCalc(home)
+            home_obj = MPCCalc(home, self.redis_url)
             self.all_homes_obj += [home_obj]
-            self.max_poss_load += home_obj.max_load   
+            self.max_poss_load += home_obj.max_load
 
     def reset_collected_data(self):
         self.timestep = 0
@@ -633,6 +668,11 @@ class Aggregator:
                 self.collected_data[home["name"]]["e_batt_opt"] = [home["battery"]["e_batt_init"]]
                 self.collected_data[home["name"]]["p_batt_ch"] = []
                 self.collected_data[home["name"]]["p_batt_disch"] = []
+            if True: # 'ev' in home["type"]:
+                self.collected_data[home["name"]]["e_ev_opt"] = ["16"]
+                self.collected_data[home["name"]]["p_ev_ch"] = []
+                self.collected_data[home["name"]]["p_ev_disch"] = []
+                self.collected_data[home["name"]]["p_v2g"] = []
 
     def check_all_data_indices(self):
         """
@@ -664,11 +704,11 @@ class Aggregator:
         """
         self.timestep = 0
 
-        self.redis_client.conn.set("start_hour_index", self.start_hour_index)
-        self.redis_client.conn.hset("current_values", "timestep", self.timestep)
+        self.redis_client.set("start_hour_index", self.start_hour_index)
+        self.redis_client.hset("current_values", "timestep", self.timestep)
 
         self.reward_price = np.zeros(self.config['agg']['rl']['action_horizon'] * self.dt)
-        self.redis_client.conn.rpush("reward_price", *self.reward_price.tolist())
+        self.redis_client.rpush("reward_price", *self.reward_price.tolist())
 
     def redis_add_all_data(self):
         """
@@ -678,21 +718,21 @@ class Aggregator:
         :return: None
         """
         for c in self.all_data.columns.to_list():
-            self.redis_client.conn.delete(c)
-            self.redis_client.conn.rpush(c, *self.all_data[c].values.tolist())
+            self.redis_client.delete(c)
+            self.redis_client.rpush(c, *self.all_data[c].values.tolist())
 
     def redis_set_current_values(self):
         """
         Sets the current values of the utility agent (reward price).
         :return: None
         """
-        self.redis_client.conn.hset("current_values", "timestep", self.timestep)
+        self.redis_client.hset("current_values", "timestep", self.timestep)
 
         if 'rl' in self.case:
             self.all_sps[self.timestep] = self.agg_setpoint
             self.all_rps[self.timestep] = self.reward_price[0]
-            self.redis_client.conn.delete("reward_price")
-            self.redis_client.conn.rpush("reward_price", *self.reward_price)
+            self.redis_client.delete("reward_price")
+            self.redis_client.rpush("reward_price", *self.reward_price)
 
     def gen_setpoint(self):
         """
@@ -756,7 +796,7 @@ class Aggregator:
         self.forecast_house_load = []
         for home in self.all_homes:
             if self.check_type == 'all' or home["type"] == self.check_type:
-                vals = self.redis_client.conn.hgetall(home["name"])
+                vals = self.redis_client.hgetall(home["name"])
                 print(home["name"])
                 for k, v in vals.items():
                     opt_keys = ["p_grid_opt", "forecast_p_grid_opt", "p_load_opt", "temp_in_opt", "temp_wh_opt", "hvac_cool_on_opt", "hvac_heat_on_opt", "wh_heat_on_opt", "cost_opt", "waterdraws", "correct_solve", "t_in_max", "t_in_min"]
@@ -764,6 +804,8 @@ class Aggregator:
                         opt_keys += ['p_pv_opt','u_pv_curt_opt']
                     if 'battery' in home["type"]:
                         opt_keys += ['p_batt_ch', 'p_batt_disch', 'e_batt_opt']
+                    if True: #'ev' in home["type"]:
+                        opt_keys += ['p_ev_ch', 'p_ev_disch', 'p_v2g', 'e_ev_opt']
                     if k in opt_keys:
                         self.collected_data[home["name"]][k].append(float(v))
                 self.house_load.append(float(vals["p_grid_opt"]))
@@ -843,12 +885,15 @@ class Aggregator:
         and the named version.
         :return: none
         """
-        date_output = os.path.join(self.outputs_dir, f"{self.start_dt.strftime('%Y-%m-%dT%H')}_{self.end_dt.strftime('%Y-%m-%dT%H')}")
-        mpc_output = os.path.join(date_output, f"{self.check_type}-homes_{self.config['community']['total_number_homes']}-horizon_{self.config['home']['hems']['prediction_horizon']}-interval_{self.dt_interval}-{self.dt_interval // self.config['home']['hems']['sub_subhourly_steps']}-solver_{self.config['home']['hems']['solver']}")
+        if not self.overwrite_output:
+            date_output = os.path.join(self.outputs_dir, f"{self.start_dt.strftime('%Y-%m-%dT%H')}_{self.end_dt.strftime('%Y-%m-%dT%H')}")
+            mpc_output = os.path.join(date_output, f"{self.check_type}-homes_{self.config['community']['total_number_homes']}-horizon_{self.config['home']['hems']['prediction_horizon']}-interval_{self.dt_interval}-{self.dt_interval // self.config['home']['hems']['sub_subhourly_steps']}-solver_{self.config['home']['hems']['solver']}")
 
-        self.run_dir = os.path.join(mpc_output, f"version-{self.version}")
-        if not os.path.isdir(self.run_dir):
-            os.makedirs(self.run_dir)
+            self.run_dir = os.path.join(mpc_output, f"version-{self.version}")
+            if not os.path.isdir(self.run_dir):
+                os.makedirs(self.run_dir)
+        else:
+            self.run_dir = self.outputs_dir
 
     def write_outputs(self):
         """
@@ -858,10 +903,13 @@ class Aggregator:
         """
         self.summarize_baseline()
 
-        case_dir = os.path.join(self.run_dir, self.case)
-        if not os.path.isdir(case_dir):
-            os.makedirs(case_dir)
-        file = os.path.join(case_dir, "results.json")
+        if not self.overwrite_output:
+            case_dir = os.path.join(self.run_dir, self.case)
+            if not os.path.isdir(case_dir):
+                os.makedirs(case_dir)
+            file = os.path.join(case_dir, "results.json")
+        else:
+            file = os.path.join(self.outputs_dir, "results.json")
         with open(file, 'w+') as f:
             json.dump(self.collected_data, f, indent=4)
 
@@ -892,8 +940,6 @@ class Aggregator:
 
     def set_dummy_rl_parameters(self):
         self.tracked_loads = self.config['community']['house_p_avg']*self.config['community']['total_number_homes']*np.ones(12)
-        # self.util = self.util_permutations[0]
-        # self.rl_params = self.rl_permutations[0]
 
     def setup_rl_agg_run(self):
         self.flush_redis()
@@ -923,7 +969,7 @@ class Aggregator:
         and home data.)
         :return: None
         """
-        self.redis_client.conn.flushall()
+        self.redis_client.flushall()
         self.log.logger.info("Flushing Redis")
         time.sleep(1)
         self.check_all_data_indices()
